<?php
namespace PME\Services;

use Doctrine\ORM\EntityManager;

/**
 * Prato Model
 * 
 * Connects to the Database to handle queries 
 * related to prato processes. 
 *  
 * @author Bruna Chaves Carneiro
 */
class PratoService
{
    protected $entityManager;
    
    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param EntityManager $entityManager Object to manage database connection and queries
     * 
     * @return void
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function __construct($entityManager)
    {   
        $this->entityManager = $entityManager;
    }
    
    public function flush(){
        $this->entityManager->flush();
    }

    public function commit(){
        $this->entityManager->commit();
    }

    public function rollback(){
        $this->entityManager->rollback();
    }
    
    public function returnVisibilityFields($cdfilial)
    {
        try {
            $sql = "SELECT
                       P.CDFILIAL,
                       P.IDQTLIQUBRUT,
                       P.IDAPRERECE,
                       P.IDGANHOCOCC,
                       P.IDAPROVCOCC,
                       P.IDPERDALIMP,
                       P.IDAPROVLIMP,
                       P.IDTEMPPREP,
                       P.IDBLOALTRELO,
                       P.IDUTREQCOMPENTD,   
                       G.IDBLOPRATLOC,
                       G.CDNVBLPRAT,
                       P.IDUTDIASLIMSEG
                    FROM
                       PARAMGERAL G, PARAMPLANEJA P
                    WHERE
                       P.CDFILIAL = :P_CDFILIAL AND
                       P.CDPARPLANEJA = '1' AND
                       G.CDPARGERAL = '0001'";
                    
            $params = array(
                "P_CDFILIAL" => $cdfilial
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaFatorConversao($produto, $sgunidade, $nrorg) 
    {
        $sql = "SELECT DISTINCT SGUNIDADE, VRFATOCONVES, CDPRODUTO, NRORG
                  FROM (SELECT SGUNIDADE, VRFATOCONVES, CDPRODUTO, NRORG
                          FROM CONVPRODUTO
                         WHERE CDPRODUTO = :CDPRODUTO
                           AND NRORG       = :NRORG
                         UNION ALL
                        SELECT SGUNIDDE         AS SGUNIDADE,
                               TO_NUMBER(VRFTCONVER) AS VRFATOCONVES,
                               :CDPRODUTO            AS CDPRODUTO,
                               NRORG
                          FROM CONVUNID
                         WHERE SGUNIDDE = :SGUNIDADE
                           AND NRORG        = :NRORG
                         UNION ALL
                        SELECT :SGUNIDADE          AS SGUNIDADE,
                               TO_NUMBER(1) AS VRFATOCONVES,
                               :CDPRODUTO               AS CDPRODUTO,
                               TO_NUMBER(:NRORG)        AS NRORG
                          FROM PARAMGERAL
                      ) CONVPRODUTO";

        $params = array(
            "CDPRODUTO" => $produto,
            "SGUNIDADE" => $sgunidade,
            "NRORG" => $nrorg
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaFatorConversaoPrato($produto, $sgunidade, $sgunidadeCompar, $nrorg){
         $sql = "SELECT TO_NUMBER(C.VRFATOCONVES) AS VRFATOCONVES,
                        RPAD('/', 1, ' ')       AS IDTIPOPE,
                        0 AS ORDEM
                   FROM CONVPRODUTO C
                   LEFT JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO
                  WHERE (C.CDPRODUTO = :P_CDPRODUTO)
                    AND (P.SGUNIDADE = :P_SGUNIDADEORI)
                    AND (C.SGUNIDADE = :P_SGUNIDADEDEST)

                  UNION ALL

                 SELECT TO_NUMBER(C.VRFATOCONVES) AS VRFATOCONVES,
                        RPAD('*', 1, ' ')       AS IDTIPOPE,
                        1 AS ORDEM
                   FROM CONVPRODUTO C
                   LEFT JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO
                  WHERE (C.CDPRODUTO = :P_CDPRODUTO)
                    AND (P.SGUNIDADE = :P_SGUNIDADEDEST)
                    AND (C.SGUNIDADE = :P_SGUNIDADEORI)

                  UNION ALL

                 SELECT TO_NUMBER(VRFTCONVER)  AS VRFATOCONVES,
                        RPAD(IDTIPOPE, 1, ' ') AS IDTIPOPE,
                        2 AS ORDEM
                   FROM CONVUNID
                  WHERE (SGUNIDPARA = :P_SGUNIDADEDEST)
                    AND (SGUNIDDE = :P_SGUNIDADEORI)

                  UNION ALL

                 SELECT TO_NUMBER(VRFTCONVER)  AS VRFATOCONVES,
                        RPAD(IDTIPOPE, 1, ' ') AS IDTIPOPE,
                        3 AS ORDEM
                   FROM CONVUNID
                  WHERE (SGUNIDPARA = :P_SGUNIDADEORI )
                    AND (SGUNIDDE = :P_SGUNIDADEDEST )

                  ORDER BY ORDEM";

        $params = array(
            "P_CDPRODUTO" => $produto,
            "P_SGUNIDADEORI" => $sgunidade,
            "P_SGUNIDADEDEST" => $sgunidadeCompar
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}